<template>
  <TransitionRoot
    :show="display"
    as="template"
  >
    <DialogRoot
      as="div"
      class="relative z-20"
      @close="close()"
    >
      <TransitionChild
        as="template"
        enter="ease-out duration-300"
        enter-from="opacity-0"
        enter-to="opacity-100"
        leave="ease-in duration-200"
        leave-from="opacity-100"
        leave-to="opacity-0"
        @after-leave="$emit('close')"
      >
        <div class="fixed inset-0 bg-gray-500 bg-opacity-75 transition-opacity" />
      </TransitionChild>
      <div class="fixed inset-0 z-20 overflow-y-auto">
        <div class="flex min-h-full items-center justify-center p-4 text-center">
          <TransitionChild
            as="template"
            enter="ease-out duration-300"
            enter-from="opacity-0 translate-y-4 sm:translate-y-0 sm:scale-95"
            enter-to="opacity-100 translate-y-0 sm:scale-100"
            leave="ease-in duration-200"
            leave-from="opacity-100 translate-y-0 sm:scale-100"
            leave-to="opacity-0 translate-y-4 sm:translate-y-0 sm:scale-95"
          >
            <DialogPanel
              :class="classes"
              class="relative w-full transform rounded-md bg-white text-left shadow-xl transition-all sm:my-8"
            >
              <template v-if="!withoutDismiss">
                <div class="absolute right-0 top-0 block pr-4 pt-4">
                  <button
                    class="rounded-md bg-white text-gray-400 hover:text-gray-500 focus:outline-none focus:ring-2 focus:ring-primary-500 focus:ring-offset-2"
                    type="button"
                    @click="close()"
                  >
                    <XMarkRegularIcon
                      aria-hidden="true"
                      class="h-6 w-6 fill-current"
                    />
                  </button>
                </div>
              </template>
              <slot />
            </DialogPanel>
          </TransitionChild>
        </div>
      </div>
    </DialogRoot>
  </TransitionRoot>
</template>

<script lang="ts" setup>
import XMarkRegularIcon from "@/svg/icons/xmark-regular.svg?component";
import { Dialog as DialogRoot, DialogPanel, TransitionChild, TransitionRoot } from "@headlessui/vue";

const props = withDefaults(
  defineProps<{
    isPage?: boolean;
    withoutDismiss?: boolean;
    size?: "md" | "lg" | "xl" | "2xl" | "3xl" | "4xl" | "5xl" | "6xl" | "7xl";
  }>(),
  {
    size: "lg",
  },
);

defineEmits<{
  close: [];
}>();

const display = ref(props.isPage ?? false);

const toggle = () => {
  if (display.value) {
    close();
    return;
  }
  open();
  return;
};

const open = () => {
  display.value = true;
};

const close = () => {
  display.value = false;
};

const classes = computed(() => ({
  "max-w-md": props.size === "md",
  "max-w-lg": props.size === "lg",
  "max-w-xl": props.size === "xl",
  "max-w-2xl": props.size === "2xl",
  "max-w-3xl": props.size === "3xl",
  "max-w-4xl": props.size === "4xl",
  "max-w-5xl": props.size === "5xl",
  "max-w-6xl": props.size === "6xl",
  "max-w-7xl": props.size === "7xl",
}));

defineExpose({ toggle, open, close, isOpen: computed<boolean>(() => !!display.value) });
</script>
